@@ -1,24 +1,25 @@
 # -*- coding: utf-8 -*-
+# 版权所有 2019 深圳米筐科技有限公司（下称“米筐科技”）
 #
-# Copyright 2017 Ricequant, Inc
+# 除非遵守当前许可，否则不得使用本软件。
 #
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+#     * 非商业用途（非商业用途指个人出于非商业目的使用本软件，或者高校、研究所等非营利机构出于教育、科研等目的使用本软件）：
+#         遵守 Apache License 2.0（下称“Apache 2.0 许可”），
+#         您可以在以下位置获得 Apache 2.0 许可的副本：http://www.apache.org/licenses/LICENSE-2.0。
+#         除非法律有要求或以书面形式达成协议，否则本软件分发时需保持当前许可“原样”不变，且不得附加任何条件。
 #
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+#     * 商业用途（商业用途指个人出于任何商业目的使用本软件，或者法人或其他组织出于任何目的使用本软件）：
+#         未经米筐科技授权，任何个人不得出于任何商业目的使用本软件（包括但不限于向第三方提供、销售、出租、出借、转让本软件、
+#         本软件的衍生产品、引用或借鉴了本软件功能或源代码的产品或服务），任何法人或其他组织不得出于任何目的使用本软件，
+#         否则米筐科技有权追究相应的知识产权侵权责任。
+#         在此前提下，对本软件的使用同样需要遵守 Apache 2.0 许可，Apache 2.0 许可与本许可冲突之处，以本许可为准。
+#         详细的授权流程，请联系 public@ricequant.com 获取。
 
 import copy
 from collections import OrderedDict
 
 from rqalpha.utils.package_helper import import_mod
-from rqalpha.utils.logger import system_log
+from rqalpha.utils.logger import system_log, system_log
 from rqalpha.utils.i18n import gettext as _
 from rqalpha.utils import RqAttrDict
 
@@ -65,22 +66,31 @@ class ModHandler(object):
 
     def start_up(self):
         for mod_name, mod_config in self._mod_list:
+            system_log.debug(_(u"mod start_up [START] {}\n{}").format(mod_name, mod_config))
             self._mod_dict[mod_name].start_up(self._env, mod_config)
+            system_log.debug(_(u"mod start_up [END]   {}").format(mod_name))
 
     def tear_down(self, *args):
         result = {}
         for mod_name, __ in reversed(self._mod_list):
-            ret = self._mod_dict[mod_name].tear_down(*args)
+            try:
+                system_log.debug(_(u"mod tear_down [START] {}").format(mod_name))
+                ret = self._mod_dict[mod_name].tear_down(*args)
+                system_log.debug(_(u"mod tear_down [END]   {}").format(mod_name))
+            except Exception as e:
+                system_log.exception("tear down fail for {}", mod_name)
+                continue
             if ret is not None:
                 result[mod_name] = ret
         return result
 
 
 SYSTEM_MOD_LIST = [
+    "sys_accounts",
     "sys_analyser",
     "sys_progress",
-    "sys_funcat",
     "sys_risk",
     "sys_simulation",
-    "sys_stock_realtime",
+    "sys_transaction_cost",
+    'sys_scheduler',
 ]
