@@ -1,48 +1,31 @@
 # -*- coding: utf-8 -*-
+# 版权所有 2019 深圳米筐科技有限公司（下称“米筐科技”）
 #
-# Copyright 2017 Ricequant, Inc
+# 除非遵守当前许可，否则不得使用本软件。
 #
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
+#     * 非商业用途（非商业用途指个人出于非商业目的使用本软件，或者高校、研究所等非营利机构出于教育、科研等目的使用本软件）：
+#         遵守 Apache License 2.0（下称“Apache 2.0 许可”），您可以在以下位置获得 Apache 2.0 许可的副本：http://www.apache.org/licenses/LICENSE-2.0。
+#         除非法律有要求或以书面形式达成协议，否则本软件分发时需保持当前许可“原样”不变，且不得附加任何条件。
 #
-#     http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+#     * 商业用途（商业用途指个人出于任何商业目的使用本软件，或者法人或其他组织出于任何目的使用本软件）：
+#         未经米筐科技授权，任何个人不得出于任何商业目的使用本软件（包括但不限于向第三方提供、销售、出租、出借、转让本软件、本软件的衍生产品、引用或借鉴了本软件功能或源代码的产品或服务），任何法人或其他组织不得出于任何目的使用本软件，否则米筐科技有权追究相应的知识产权侵权责任。
+#         在此前提下，对本软件的使用同样需要遵守 Apache 2.0 许可，Apache 2.0 许可与本许可冲突之处，以本许可为准。
+#         详细的授权流程，请联系 public@ricequant.com 获取。
 
 import datetime
+from typing import Union
 from collections import namedtuple
 
-from .py2 import lru_cache
+import six
+from dateutil.parser import parse
+
+from rqalpha.utils.py2 import lru_cache
+from rqalpha.utils.exception import RQInvalidArgument
 
 
 TimeRange = namedtuple('TimeRange', ['start', 'end'])
 
 
-def get_month_begin_time(time=None):
-    if time is None:
-        time = datetime.datetime.now()
-    return time.replace(day=1, hour=0, minute=0, second=0, microsecond=0)
-
-
-def get_month_end_time(time=None):
-    try:
-        return time.replace(month=time.month + 1, day=1, hour=23, minute=59, second=59,
-                            microsecond=999) - datetime.timedelta(days=1)
-    except ValueError:
-        return time.replace(year=time.year + 1, month=1, day=1, hour=23, minute=59, second=59,
-                            microsecond=999) - datetime.timedelta(days=1)
-
-
-def get_last_date(trading_calendar, dt):
-    idx = trading_calendar.searchsorted(dt)
-    return trading_calendar[idx - 1]
-
-
 def convert_date_to_date_int(dt):
     t = dt.year * 10000 + dt.month * 100 + dt.day
     return t
@@ -85,6 +68,12 @@ def convert_int_to_datetime(dt_int):
     return datetime.datetime(year, month, day, hour, minute, second)
 
 
+def convert_ms_int_to_datetime(ms_dt_int):
+    dt_int, ms_int = divmod(ms_dt_int, 1000)
+    dt = convert_int_to_datetime(dt_int).replace(microsecond=ms_int * 1000)
+    return dt
+
+
 def convert_date_time_ms_int_to_datetime(date_int, time_int):
     date_int, time_int = int(date_int), int(time_int)
     dt = _convert_int_to_date(date_int)
@@ -95,3 +84,16 @@ def convert_date_time_ms_int_to_datetime(date_int, time_int):
 
     return dt.replace(hour=hours, minute=minutes, second=seconds,
                       microsecond=millisecond * 1000)
+
+
+def to_date(date):
+    # type: (Union[str, datetime.date, datetime.datetime]) -> datetime.date
+    if isinstance(date, six.string_types):
+        return parse(date).date()
+    elif isinstance(date, datetime.date):
+        return date
+    elif isinstance(date, datetime.datetime):
+        return date.date()
+    else:
+        raise RQInvalidArgument("unknown date value: {}".format(date))
+
